#!/bin/bash

#
# This script has been used to generate and classify use cases for
# static lists' testing as can be seen in runtest.sh.  Refer to that
# file for more details.
#



list_possible_DeltaABI() {
    #
    # All known ABI "fates"
    #
    # All possibilities what can happen to ABI within a package.
    # In case ABI is moved to another package, we handle it elsewhere
    #
    echo none
    echo added          # s00d10
    echo kept           # s10d10
    echo downgraded     # s01d10
    echo removed        # s10d00
    echo bumped         # s10d01
}

mkhint() {
    #
    # From fate parameter
    #
    case "$Type" in
        abi)
            case $DAbi in
                s00d??) echo added      ;;
                s10d01) echo bumped     ;;
                s01d10) echo downgraded ;;
                s01d01) echo kept       ;;
                s01d11) echo kept       ;;
                s10d10) echo kept       ;;
                s10d11) echo kept       ;;
                s??d00) echo removed    ;;
                *)      rlFail "unsupported version delta: $DAbi" ;;
            esac
            ;;
        gen)
            case $DRepo in
                none)
                    case $DPkg in
                        0)  echo 
                    esac
                    ;;
                *)
                    echo moved$DRepo
                    ;;
            esac
    esac
}

possible_fates() {
    #
    # Create matrix of all (im)possible cases
    #
    local Type Req DRepo DPkg DAbi
    #shellcheck disable=SC2034
    for Type in abi gen;
    do
        for DRepo in none equiv different;
        do
            for DPkg in none replaced obsoleted removed;
            do
                for DAbi in $(list_possible_DeltaABI);
                do
                    for Req in none our their;
                    do
                        echo "Type=$Type,DRepo=$DRepo,DPkg=$DPkg,DAbi=$DAbi,Req=$Req"
                    done
                done
            done
        done
    done
}

validate_fate() {
    #
    # Take fate per line on stdin, print only valid combinations
    #
    local ok=false
    case $DRepo:$DPkg:$(mkhint) in
        non:0:added)      ok=true ;;
        non:0:removed)    ok=true ;;
        *:*:added)        ok=false ;;
        *:*:removed)      ok=false ;;
        *)                ok=true ;;
    esac
    $ok
}

hide_invalid() {
    #
    # Hide invalid cases
    #
    grep -v \
      -e 'Type=gen.*DAbi=[^n]' \
      -e 'Type=abi.*DAbi=none' \
      -e 'Type=abi.*DRepo=[^n]' \
      -e 'Type=abi.*DPkg=[^n]' \
      -e 'Type=gen.*DRepo=[^n].*DPkg=removed'
}

hide_boring() {
    #
    # Hide cases that don't produce any meaningful behavior
    #
    grep -v \
      -e 'Req=none' \
      -e 'DAbi=added'
}

possible_fates | hide_invalid | hide_boring
